@@ -1,5 +1,5 @@
 /* events.c -- Event handling
-   $Id: events.c,v 1.129 2000/11/13 23:57:43 jsh Exp $
+   $Id: events.c,v 1.135 2001/01/05 19:05:33 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -51,6 +51,8 @@ XEvent *current_x_event;
 static bool current_event_updated_mouse;
 static repv current_event_window;
 
+static repv saved_current_context_map;
+
 /* We need a ButtonRelease on this fp. */
 struct frame_part *clicked_frame_part;
 
@@ -103,8 +105,6 @@ DEFSYM(normal, "normal");
 DEFSYM(grab, "grab");
 DEFSYM(ungrab, "ungrab");
 
-DEFSYM(ignore_fp_keymap, "ignore-fp-keymap");
-
 repv Fsynthetic_configure_mutex (repv);
 
 /* `Time' will always be 32-bits, due to underlying wire protocol (?) */
@@ -326,22 +326,30 @@ synthesize_button_release (void)
 static repv
 current_context_map (void)
 {
-    repv map = Qnil;
-
-    /* Only use the context map if the frame part is currently clicked,
-       and it's window is visible (i.e. not iconified) */
-    if (clicked_frame_part
-	&& clicked_frame_part->clicked
-	&& clicked_frame_part->win != 0
-	&& clicked_frame_part->win->visible)
+    if (saved_current_context_map == rep_NULL)
     {
-	repv tem = Fwindow_get (rep_VAL (clicked_frame_part->win),
-				Qignore_fp_keymap);
-	if (tem == Qnil)
+	repv map = Qnil;
+
+	/* Only use the context map if the frame part is currently clicked,
+	   and it's window is visible (i.e. not iconified) */
+	if (clicked_frame_part
+	    && clicked_frame_part->clicked
+	    && clicked_frame_part->win != 0
+	    && clicked_frame_part->win->visible)
+	{
 	    map = get_keymap_for_frame_part (clicked_frame_part);
+	}
+
+	saved_current_context_map = map;
     }
 
-    return map;
+    return saved_current_context_map;
+}
+
+static void
+flush_current_context_map (void)
+{
+    saved_current_context_map = rep_NULL;
 }
 
 static void
@@ -350,6 +358,13 @@ button_press (XEvent *ev)
     struct frame_part *fp;
     Lisp_Window *w = 0;
 
+    /* This works for both press and release events. The desired outcome
+       is that press and motion events get the context map active when
+       the button was initially pressed, while release events get
+       the context map active when the button was released (in case the
+       pointer left the frame part) */
+    flush_current_context_map ();
+
     record_mouse_position (ev->xbutton.x_root, ev->xbutton.y_root,
 			   ev->type, ev->xany.window);
 
@@ -412,6 +427,11 @@ motion_notify (XEvent *ev)
 	eval_input_event (current_context_map ());
 
     XAllowEvents (dpy, SyncPointer, last_event_time);
+
+    /* Don't call flush_current_context_map (), since we want to
+       fix it for all motion events (in case the motion threshold
+       kicks in, and the pointer leaves the original fp before
+       a motion event is actually evaluated) */
 }
 
 static void
@@ -583,7 +603,7 @@ client_message (XEvent *ev)
     case 32:
 	data = Fmake_vector (rep_MAKE_INT(5), Qnil);
 	for (i = 0; i < 5; i++)
-	    rep_VECTI(data,i) = rep_MAKE_INT (ev->xclient.data.l[i]);
+	    rep_VECTI(data,i) = rep_make_long_uint (ev->xclient.data.l[i]);
 	break;
 
     default:
@@ -655,12 +675,14 @@ map_request (XEvent *ev)
 
     if (!w->client_unmapped)
 	XMapWindow (dpy, w->id);
-    else
-	/* wouldn't happen otherwise */
-	Fcall_window_hook (Qmap_notify_hook, rep_VAL(w), Qnil, Qnil);
 
     if (w->visible)
 	XMapWindow (dpy, w->frame);
+
+    if (w->client_unmapped)
+	/* wouldn't happen otherwise */
+	Fcall_window_hook (Qmap_notify_hook, rep_VAL(w), Qnil, Qnil);
+
 }
 
 static void
@@ -900,6 +922,13 @@ focus_out (XEvent *ev)
 	    }
 	}
     }
+    else if ((w = x_find_window_by_id (ev->xunmap.window)) != 0
+	     && w->id == 0 && ev->xunmap.window == w->saved_id)
+    {
+	/* focus-out event from a deleted window */
+	if (focus_window == w)
+	    focus_window = 0;
+    }
 }
 
 static void
@@ -1276,7 +1305,6 @@ the server, otherwise it's taken from the current event (if possible).
 	{
 	    record_mouse_position (x, y, -1, 0);
 	}
-	emit_pending_destroys ();
     }
     return Fcons (rep_MAKE_INT(current_mouse_x),
 		  rep_MAKE_INT(current_mouse_y));
@@ -1337,7 +1365,6 @@ is in the root window.
 
     XQueryPointer (dpy, root_window, &root, &child,
 		   &current_mouse_x, &current_mouse_y, &win_x, &win_y, &state);
-    emit_pending_destroys ();
     if (child != 0)
     {
 	Lisp_Window *w = find_window_by_id (child);
@@ -1587,9 +1614,8 @@ events_init (void)
     rep_INTERN(grab);
     rep_INTERN(ungrab);
 
-    rep_INTERN(ignore_fp_keymap);
-
     rep_mark_static (&current_event_window);
+    rep_mark_static (&saved_current_context_map);
 
     event_handler_context = XUniqueContext ();
 
