@@ -1,5 +1,5 @@
 /* functions.c -- useful window manager Lisp functions
-   $Id: functions.c,v 1.83 2000/10/23 04:09:07 john Exp $
+   $Id: functions.c,v 1.85 2000/11/27 18:06:27 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -545,7 +545,6 @@ WINDOW may be the symbol `root', a window object or a numeric window id.
     Atom *atoms;
     int count;
     repv ret = Qnil;
-    rep_GC_root gc_ret;
 
     w = x_win_from_arg (win);
     if (w == 0)
@@ -566,13 +565,7 @@ WINDOW may be the symbol `root', a window object or a numeric window id.
 	}
 	XFree (atoms);
     }
-    ret = Fnreverse (ret);
-
-    rep_PUSHGC(gc_ret, ret);
-    emit_pending_destroys ();
-    rep_POPGC;
-
-    return ret;
+    return Fnreverse (ret);
 }
 
 DEFUN("get-x-property", Fget_x_property, Sget_x_property,
@@ -602,7 +595,6 @@ symbols, representing the atoms read.
     u_long nitems;
     u_char *data = 0;
     repv type_sym, ret_data = Qnil;
-    rep_GC_root gc_ret_data;
 
     w = x_win_from_arg (win);
     rep_DECLARE2(prop, rep_SYMBOLP);
@@ -660,19 +652,13 @@ symbols, representing the atoms read.
 	    if (type == XA_ATOM && (name = x_atom_symbol (l_data[i])) != Qnil)
 		rep_VECTI(ret_data, i) = name;
 	    else
-		rep_VECTI(ret_data, i) = rep_MAKE_INT(l_data[i]);
+		rep_VECTI(ret_data, i) = rep_make_long_uint(l_data[i]);
 	}
 	break;
     }
     XFree (data);
 
-    ret_data = rep_list_3 (type_sym, rep_MAKE_INT(format), ret_data);
-
-    rep_PUSHGC(gc_ret_data, ret_data);
-    emit_pending_destroys ();
-    rep_POPGC;
-
-    return ret_data;
+    return rep_list_3 (type_sym, rep_MAKE_INT(format), ret_data);
 }
 
 DEFUN("set-x-property", Fset_x_property, Sset_x_property,
