@@ -1,5 +1,5 @@
 /* windows.c -- window manipulation
-   $Id: windows.c,v 1.104 2000/11/13 19:36:55 jsh Exp $
+   $Id: windows.c,v 1.108 2001/01/18 19:47:34 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -22,6 +22,11 @@
 #include "sawmill.h"
 #include <X11/extensions/shape.h>
 
+/* Work around for X11R5 and earlier */
+#ifndef XUrgencyHint
+#define XUrgencyHint (1 << 8)
+#endif
+
 Lisp_Window *window_list;
 int window_type;
 
@@ -77,6 +82,14 @@ DEFSYM(south_east, "south-east");
 
 static repv gravity_map[StaticGravity+1];
 
+struct prop_handler {
+    struct prop_handler *next;
+    repv prop;
+    void (*callback) (Lisp_Window *w, repv prop, repv old, repv new);
+};
+
+static struct prop_handler *prop_handlers;
+
 
 /* utilities */
 
@@ -433,9 +446,6 @@ remove_window (Lisp_Window *w, repv destroyed, repv from_error)
     DB(("remove_window (%s, %s)\n",
 	rep_STR(w->name), destroyed == Qnil ? "nil" : "t"));
 
-    if (focus_window == w)
-	focus_window = 0;
-
     if (w->id != 0)
     {
 	if (destroyed == Qnil && from_error == Qnil)
@@ -448,13 +458,8 @@ remove_window (Lisp_Window *w, repv destroyed, repv from_error)
 	}
 
 	if (from_error == Qnil)
-	{
 	    destroy_window_frame (w, FALSE);
 
-	    if (focus_window == w)
-		focus_on_window (0);
-	}
-
 	w->id = 0;
 	pending_destroys++;
 
@@ -569,6 +574,17 @@ object WINDOW.
     return ret;
 }
 
+void
+register_property_monitor (repv prop,
+			   void (*callback) (Lisp_Window *, repv, repv, repv))
+{
+    struct prop_handler *ph = rep_alloc (sizeof (struct prop_handler));
+    ph->next = prop_handlers;
+    prop_handlers = ph;
+    ph->prop = prop;
+    ph->callback = callback;
+}    
+
 DEFUN("window-put", Fwindow_put, Swindow_put,
       (repv win, repv prop, repv val), rep_Subr3) /*
 ::doc:sawfish.wm.windows.subrs#window-put::
@@ -588,13 +604,12 @@ Note that these are Lisp properties not X properties.
 	    || (!rep_SYMBOLP(prop)
 		&& rep_value_cmp (rep_CAR(plist), prop) == 0))
 	{
-	    if (prop == Qkeymap && VWIN(win)->id != 0)
+	    struct prop_handler *ph;
+	    for (ph = prop_handlers; ph != 0; ph = ph->next)
 	    {
-		/* A bit of a hack */
-		grab_keymap_events (VWIN(win)->id,
-				    rep_CAR(rep_CDR(plist)), FALSE);
-		grab_keymap_events (VWIN(win)->id, val, TRUE);
-
+		repv old = rep_CADR (plist);
+		if (ph->prop == prop && old != val)
+		    ph->callback (VWIN (win), prop, old, val);
 	    }
 	    rep_CAR(rep_CDR(plist)) = val;
 	    return val;
@@ -872,7 +887,6 @@ client windows.
     {
 	int i;
 	repv ret = Qnil;
-	rep_GC_root gc_ret;
 	for (i = 0; i < nchildren; i++)
 	{
 	    Lisp_Window *w = find_window_by_id (children[i]);
@@ -881,9 +895,6 @@ client windows.
 	}
 	if (children != 0)
 	    XFree (children);
-	rep_PUSHGC(gc_ret, ret);
-	emit_pending_destroys ();
-	rep_POPGC;
 	return ret;
     }
     else
@@ -1208,33 +1219,57 @@ WINDOW. Returns the symbol `nil' if no such image.
 
    if (VWIN (win)->icon_image == rep_NULL)
    {
-       if (VWIN (win)->wmhints == 0)
-	   VWIN (win)->icon_image = Qnil;
-       else
-       {
-	   repv id, mask_id;
-	   repv ret = Qnil;
+       Window pixmap_id = 0, mask_id = 0;
 
+       if (VWIN (win)->wmhints != 0)
+       {
 	   if (VWIN (win)->wmhints->flags & IconPixmapHint
 	       && VWIN (win)->wmhints->icon_pixmap != 0)
 	   {
-	       id = rep_MAKE_INT (VWIN (win)->wmhints->icon_pixmap);
+	       pixmap_id = VWIN (win)->wmhints->icon_pixmap;
 	   }
-	   else
-	       id = Qnil;
 
 	   if (VWIN (win)->wmhints->flags & IconMaskHint
 	       && VWIN (win)->wmhints->icon_mask != 0)
 	   {
-	       mask_id = rep_MAKE_INT (VWIN (win)->wmhints->icon_mask);
+	       mask_id = VWIN (win)->wmhints->icon_mask;
 	   }
-	   else
-	       mask_id = Qnil;
+       }
 
-	   if (id != Qnil)
-	       ret = Fmake_image_from_x_drawable (id, mask_id);
+       if (pixmap_id == 0 && VWIN (win)->id != 0)
+       {
+	   Atom actual_type;
+	   int actual_format;
+	   long nitems, bytes_after;
+	   u_long *data = 0;
 
-	   VWIN (win)->icon_image = ret;
+	   static Atom kwm_win_icon = 0;
+
+	   if (kwm_win_icon == 0)
+	       kwm_win_icon = XInternAtom (dpy, "KWM_WIN_ICON", False);
+
+	   if (XGetWindowProperty (dpy, VWIN (win)->id, kwm_win_icon,
+				   0, 2, False, kwm_win_icon,
+				   &actual_type, &actual_format,
+				   &nitems, &bytes_after,
+				   (u_char **) &data) == Success
+	       && actual_type == kwm_win_icon
+	       && bytes_after == 0)
+	   {
+	       pixmap_id = data[0];
+	       mask_id = data[1];
+	   }
+	   if (data != 0)
+	       XFree (data);
+       }
+
+       VWIN (win)->icon_image = Qnil;
+
+       if (pixmap_id != 0)
+       {
+	   VWIN (win)->icon_image = (Fmake_image_from_x_drawable
+				     (rep_MAKE_INT (pixmap_id),
+				      mask_id ? rep_MAKE_INT (mask_id) : Qnil));
        }
    }
 
@@ -1334,13 +1369,16 @@ window_mark (repv win)
 static void
 window_mark_type (void)
 {
-    Lisp_Window *w = window_list;
-    while (w != 0)
+    Lisp_Window *w;
+    struct prop_handler *ph;
+    for (w = window_list; w != 0; w = w->next)
     {
 	if (w->id != 0 || !w->destroyed)
 	    rep_MARKVAL(rep_VAL(w));
-	w = w->next;
     }
+    for (ph = prop_handlers; ph != 0; ph = ph->next)
+	rep_MARKVAL (ph->prop);
+    rep_MARKVAL (rep_VAL (focus_window));
 }
 
 static void
